@@ -1,27 +1,43 @@
 import abc
+import json
 import logging
 import os
-from typing import Dict, Any, Callable, Tuple
+from typing import Dict, Any, Callable, Tuple, List, Generator
 
 from google import auth
+from google.auth import iam
 from google.auth.credentials import Credentials
 from google.auth.impersonated_credentials import Credentials as ImpersonatedCredentials
+from google.auth.transport import requests
+from google.oauth2.service_account import Credentials as ServiceAccountCredentials
+from google.protobuf.duration_pb2 import Duration
 from googleapiclient.discovery import build
+from googleapiclient.errors import HttpError
 
-DEFAULT_PROJECT_ID = os.environ.get('PROJECT_ID')
-DEFAULT_LOCATION = os.environ.get('LOCATION', None)
+from gcp_pilot import exceptions
+
+DEFAULT_LOCATION = os.environ.get('GOOGLE_CLOUD_LOCATION', None)
+TOKEN_URI = 'https://accounts.google.com/o/oauth2/token'
 
 PolicyType = Dict[str, Any]
 AuthType = Tuple[Credentials, str]
 ImpersonatedAuthType = Tuple[ImpersonatedCredentials, str]
+ResourceType = Dict[str, Any]
 
 logger = logging.getLogger()
 
+_CACHED_LOCATIONS = {}  # TODO: Implement a smarter solution for caching project's location
+
 
 def _get_project_default_location(credentials, project_id: str, default_zone: str = '1') -> str:
-    service = build(serviceName='appengine', version='v1', credentials=credentials, cache_discovery=False)
-    data = service.apps().get(appsId=project_id).execute()
-    return data['locationId'] + default_zone
+    location = _CACHED_LOCATIONS.get(project_id, None)
+
+    if not location:
+        service = build(serviceName='appengine', version='v1', credentials=credentials, cache_discovery=False)
+        data = service.apps().get(appsId=project_id).execute()
+        location = data['locationId'] + default_zone
+        _CACHED_LOCATIONS[project_id] = location
+    return location
 
 
 MINIMAL_SCOPES = [
@@ -31,9 +47,9 @@ MINIMAL_SCOPES = [
 
 class GoogleCloudPilotAPI(abc.ABC):
     _client_class = None
-    _scopes = []
-    _iam_roles = []
-    _cached_credentials = None
+    _scopes: List[str] = []
+    _iam_roles: List[str] = []
+    _cached_credentials: AuthType = None
 
     def __init__(
             self,
@@ -56,7 +72,7 @@ class GoogleCloudPilotAPI(abc.ABC):
         )
 
     def _set_project_id(self, project_id: str, credential_project_id: str) -> str:
-        return project_id or DEFAULT_PROJECT_ID or credential_project_id
+        return project_id or credential_project_id
 
     def _set_location(self, location: str = None) -> str:
         return location or DEFAULT_LOCATION or _get_project_default_location(
@@ -65,7 +81,12 @@ class GoogleCloudPilotAPI(abc.ABC):
         )
 
     @classmethod
-    def _impersonate_account(cls, credentials, service_account, scopes) -> ImpersonatedAuthType:
+    def _impersonate_account(
+            cls,
+            credentials: Credentials,
+            service_account: str,
+            scopes: List[str],
+    ) -> ImpersonatedAuthType:
         credentials = ImpersonatedCredentials(
             source_credentials=credentials,
             target_principal=service_account,
@@ -80,27 +101,62 @@ class GoogleCloudPilotAPI(abc.ABC):
 
         return credentials, project_id
 
+    @classmethod
+    def _delegated_credential(
+            cls,
+            credentials: Credentials,
+            subject: str,
+            scopes: List[str],
+    ) -> ServiceAccountCredentials:
+        try:
+            admin_credentials = credentials.with_subject(subject).with_scopes(scopes)
+        except AttributeError:
+            # When inside GCP, the credentials provided by the metadata service are immutable.
+            # https://github.com/GoogleCloudPlatform/professional-services/tree/master/examples/gce-to-adminsdk
+
+            request = requests.Request()
+            credentials.refresh(request)
+
+            # Create an IAM signer using the bootstrap credentials.
+            signer = iam.Signer(
+                request,
+                credentials,
+                credentials.service_account_email,
+            )
+            # Create OAuth 2.0 Service Account credentials using the IAM-based
+            # signer and the bootstrap_credential's service account email.
+            admin_credentials = ServiceAccountCredentials(
+                signer,
+                credentials.service_account_email,
+                TOKEN_URI,
+                scopes=scopes,
+                subject=subject,
+            )
+
+        return admin_credentials
+
     @classmethod
     def _set_credentials(cls, subject: str = None, impersonate_account: str = None) -> AuthType:
         # Speed up consecutive authentications
         # TODO: check if this does not break multiple client usage
+        all_scopes = MINIMAL_SCOPES + cls._scopes
         if not cls._cached_credentials:
-            all_scopes = MINIMAL_SCOPES + cls._scopes
             credentials, project_id = auth.default(scopes=all_scopes)
-
-            if impersonate_account:
-                credentials, impersonated_project_id = cls._impersonate_account(
-                    credentials=credentials,
-                    service_account=impersonate_account,
-                    scopes=all_scopes,
-                )
-                project_id = impersonated_project_id or project_id
-
             cls._cached_credentials = credentials, project_id
+        else:
+            credentials, project_id = cls._cached_credentials
+
+        if impersonate_account:
+            credentials, impersonated_project_id = cls._impersonate_account(
+                credentials=credentials,
+                service_account=impersonate_account,
+                scopes=all_scopes,
+            )
+            project_id = impersonated_project_id or project_id
 
-        credentials, project_id = cls._cached_credentials
         if subject:
-            credentials = credentials.with_subject(subject=subject)
+            credentials = cls._delegated_credential(credentials=credentials, subject=subject, scopes=all_scopes)
+
         return credentials, (project_id or getattr(credentials, 'project_id'))
 
     @property
@@ -108,36 +164,23 @@ class GoogleCloudPilotAPI(abc.ABC):
         return {'oidc_token': {'service_account_email': self.credentials.service_account_email}}
 
     async def add_permissions(self, email: str, project_id: str = None) -> None:
-        from gcp_pilot.resource import GoogleResourceManager  # pylint: disable=import-outside-toplevel
+        from gcp_pilot.resource import ResourceManager  # pylint: disable=import-outside-toplevel
 
         for role in self._iam_roles:
-            await GoogleResourceManager().add_member(
+            await ResourceManager().add_member(
                 email=email,
                 role=role,
                 project_id=project_id or self.project_id,
             )
 
     def _get_project_number(self, project_id: str) -> int:
-        from gcp_pilot.resource import GoogleResourceManager  # pylint: disable=import-outside-toplevel
+        from gcp_pilot.resource import ResourceManager  # pylint: disable=import-outside-toplevel
 
-        project = GoogleResourceManager().get_project(project_id=project_id)
+        project = ResourceManager().get_project(project_id=project_id)
         return project.projectNumber
 
-    def _paginate(self, method: Callable, result_key: str, params: Dict[str, Any] = None) -> Dict[str, Any]:
-        page_token = None
-        params = params or {}
-
-        while True:
-            results = method(
-                **params,
-                pageToken=page_token,
-            ).execute()
-            for item in results.get(result_key, []):
-                yield item
-
-            page_token = results.get('nextPageToken')
-            if not page_token:
-                break
+    def _as_duration(self, seconds):
+        return Duration(seconds=seconds)
 
 
 class AccountManagerMixin:
@@ -191,3 +234,63 @@ class AppEngineBasedService:
                 f"It uses App Engine's default location for your project: {project_location}"
             )
         return project_location
+
+
+def friendly_http_error(func):
+    _reasons = {
+        'notFound': exceptions.NotFound,
+        'deleted': exceptions.AlreadyDeleted,
+        'forbidden': exceptions.NotAllowed,
+    }
+
+    def inner_function(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except HttpError as exc:
+            error = json.loads(exc.content)['error']['errors'][0]
+
+            exception_klass = _reasons.get(error['reason'], None)
+            if exception_klass:
+                raise exception_klass() from exc  # TODO: add more details to the exceptions
+            raise exc
+
+    return inner_function
+
+
+class DiscoveryMixin:
+    @friendly_http_error
+    def _execute(self, method: Callable, **kwargs) -> ResourceType:
+        return method(**kwargs).execute()
+
+    def _paginate(
+            self,
+            method: Callable,
+            result_key: str = 'items',
+            params: Dict[str, Any] = None,
+            order_by: str = None,
+            limit: int = 200,
+    ) -> Generator[ResourceType, None, None]:
+        page_token = None
+        params = params or {}
+
+        if order_by:
+            if order_by.startswith('-'):
+                params['sortOrder'] = 'DESCENDING'
+                order_by = order_by[1:]
+            params['orderBy'] = order_by
+
+        if limit:
+            params['maxResults'] = limit
+
+        while True:
+            results = self._execute(
+                method=method,
+                **params,
+                pageToken=page_token,
+            )
+            for item in results.get(result_key, []):
+                yield item
+
+            page_token = results.get('nextPageToken')
+            if not page_token:
+                break
