@@ -1,16 +1,19 @@
 # More Information: https://cloud.google.com/cloud-build/docs/api
 from dataclasses import dataclass, field
-from typing import List, Dict, Any
+from typing import List, Dict, Any, Union
+from urllib.parse import urlparse
 
 from google.api_core.exceptions import AlreadyExists
 from google.cloud.devtools import cloudbuild_v1
 
+from gcp_pilot import exceptions
 from gcp_pilot.base import GoogleCloudPilotAPI
 
 TriggerType = cloudbuild_v1.BuildTrigger
+AnyEventType = Union[cloudbuild_v1.GitHubEventsConfig, cloudbuild_v1.RepoSource]
 
 
-class GoogleCloudBuild(GoogleCloudPilotAPI):
+class CloudBuild(GoogleCloudPilotAPI):
     _client_class = cloudbuild_v1.CloudBuildClient
 
     def make_build_step(
@@ -29,32 +32,87 @@ class GoogleCloudBuild(GoogleCloudPilotAPI):
             entrypoint=entrypoint,
         )
 
+    def make_source_repo_event(
+            self,
+            repo_name: str,
+            branch_name: str = None,
+            tag_name: str = None,
+            project_id: str = None,
+    ) -> cloudbuild_v1.RepoSource:
+        if not branch_name and not tag_name:
+            branch_name = 'master'
+
+        params = {}
+        if branch_name:
+            params['branch_name'] = branch_name
+        if tag_name:
+            params['tag_name'] = tag_name
+        return cloudbuild_v1.RepoSource(
+            project_id=project_id or self.project_id,
+            repo_name=repo_name,
+            **params,
+        )
+
+    def make_github_event(
+            self,
+            url,
+            branch_name: str = None,
+            tag_name: str = None,
+    ) -> cloudbuild_v1.GitHubEventsConfig:
+        if not branch_name and not tag_name:
+            branch_name = 'master'
+
+        params = {}
+        if branch_name:
+            params['branch'] = branch_name
+        if tag_name:
+            params['tag'] = tag_name
+
+        path = urlparse(url).path
+        owner, name = path.split('/')[1:]
+        return cloudbuild_v1.GitHubEventsConfig(
+            owner=owner,
+            name=name,
+            push=cloudbuild_v1.PushFilter(
+                **params,
+            ),
+        )
+
     def _make_trigger(
             self,
             name: str,
             description: str,
-            repo_name: str,
             steps: List[cloudbuild_v1.BuildStep],
-            branch_name: str,
+            event: AnyEventType,
             tags: List[str],
-            project_id: str,
             images: List[str] = None,
             substitutions: Dict[str, str] = None,
     ) -> cloudbuild_v1.BuildTrigger:
+
+        def _get_event_param():
+            valid_events = {
+                'trigger_template': cloudbuild_v1.RepoSource,
+                'github': cloudbuild_v1.GitHubEventsConfig,
+            }
+            for key, klass in valid_events.items():
+                if isinstance(event, klass):
+                    return key
+            raise exceptions.ValidationError(f"Unsupported event type {event.__class__.__name__,}")
+
+        params = {
+            _get_event_param(): event
+        }
+
         return cloudbuild_v1.BuildTrigger(
             name=name,
             description=description,
             tags=tags,
-            trigger_template=cloudbuild_v1.RepoSource(
-                project_id=project_id or self.project_id,
-                repo_name=repo_name,
-                branch_name=branch_name,
-            ),
             build=cloudbuild_v1.Build(
                 steps=steps,
                 images=images or [],
             ),
             substitutions=substitutions,
+            **params,
         )
 
     async def get_trigger(self, trigger_id: str, project_id: str = None) -> TriggerType:
@@ -75,9 +133,8 @@ class GoogleCloudBuild(GoogleCloudPilotAPI):
             self,
             name: str,
             description: str,
-            repo_name: str,
+            event: AnyEventType,
             steps: List[cloudbuild_v1.BuildStep],
-            branch_name: str = 'master',
             tags: List[str] = None,
             project_id: str = None,
             images: List[str] = None,
@@ -86,11 +143,9 @@ class GoogleCloudBuild(GoogleCloudPilotAPI):
         trigger = self._make_trigger(
             name=name,
             description=description,
-            repo_name=repo_name,
+            event=event,
             steps=steps,
-            branch_name=branch_name,
             tags=tags,
-            project_id=project_id or self.project_id,
             images=images,
             substitutions=substitutions,
         )
@@ -105,22 +160,19 @@ class GoogleCloudBuild(GoogleCloudPilotAPI):
             self,
             name: str,
             description: str,
-            repo_name: str,
+            event: AnyEventType,
             steps: List[cloudbuild_v1.BuildStep],
-            branch_name: str = 'master',
             tags: List[str] = None,
-            project_id: str = None,
             images: List[str] = None,
             substitutions: Dict[str, str] = None,
+            project_id: str = None,
     ) -> TriggerType:
         trigger = self._make_trigger(
             name=name,
             description=description,
-            repo_name=repo_name,
+            event=event,
             steps=steps,
-            branch_name=branch_name,
             tags=tags,
-            project_id=project_id or self.project_id,
             images=images,
             substitutions=substitutions,
         )
@@ -136,9 +188,8 @@ class GoogleCloudBuild(GoogleCloudPilotAPI):
             self,
             name: str,
             description: str,
-            repo_name: str,
+            event: AnyEventType,
             steps: List[cloudbuild_v1.BuildStep],
-            branch_name: str = 'master',
             tags: List[str] = None,
             project_id: str = None,
             images: List[str] = None,
@@ -147,9 +198,8 @@ class GoogleCloudBuild(GoogleCloudPilotAPI):
         create_args = dict(
             name=name,
             description=description,
-            repo_name=repo_name,
+            event=event,
             steps=steps,
-            branch_name=branch_name,
             tags=tags,
             project_id=project_id,
             images=images,
